/*
* Copyright 2014 bitbone AG
*
* http://www.bitbone.de/
*
* All rights reserved.
*
* The source code of this program is made available
* under the terms of the GNU Affero General Public License version 3
* (GNU AGPL V3) as published by the Free Software Foundation.
*
* Binary versions of this program provided by Univention to you as
* well as other copyrighted, protected or trademarked materials like
* Logos, graphics, fonts, specific documentations and configurations,
* cryptographic keys etc. are subject to a license agreement between
* you and Univention and not subject to the GNU AGPL V3.
*
* In the case you use this program under the terms of the GNU AGPL V3,
* the program is provided in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Affero General Public License for more details.
*
* You should have received a copy of the GNU Affero General Public
* License with the Debian GNU/Linux or Univention distribution in file
* /usr/share/common-licenses/AGPL-3; if not, see
* <http://www.gnu.org/licenses/>.
*/
/*global define*/

define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/on",
    "dojo/when",
    "dojo/topic",
    "umc/tools",
    "umc/dialog",
    "umc/widgets/Grid",
    "umc/widgets/Page",
    "umc/widgets/Form",
    "umc/widgets/Module",
    "umc/widgets/Text",
    "umc/widgets/PasswordInputBox",
    "umc/widgets/CheckBox",
    "umc/widgets/Page",
    "umc/i18n!umc/modules/klms"
    ], function(declare, lang, on, when, topic, tools, dialog, Grid, Page, Form, Module, Text, PasswordInputBox, CheckBox, Page, _) {
        return declare("umc.modules.klms", [ Module ], {
            idProperty: 'id',

            _previousStatus: false,
            _hasChanges: false,

            postMixInProperties: function() {
                // is called after all inherited properties/methods have been mixed
                // into the object (originates from dijit._Widget)

                // it is important to call the parent's postMixInProperties() method
                this.inherited(arguments);

                // Set the opacity for the standby animation to 100% in order to mask
                // GUI changes when the module is opened. Call this.standby(true|false)
                // to enabled/disable the animation.
                this.standbyOpacity = 1;
            },

            buildRendering: function() {
                // is called after all DOM nodes have been setup
                // (originates from dijit._Widget)

                // it is important to call the parent's postMixInProperties() method
                this.inherited(arguments);

                // start the standby animation in order prevent any interaction before the
                // form values are loaded
                this.standby(true);

                // render the page containing search form and grid
                this.renderMain();
            },

            renderMain: function(containers, superordinates) {

                var ucrDeferred = tools.ucr(["klms/enabled"]).then(function(ucr) {
                    return tools.isTrue(ucr["klms/enabled"]);
                });


                when(ucrDeferred, lang.hitch(this, function(enabled) {
                    this._previousStatus = enabled;

                    this._mainPage = new Page({
                        headerButtons: [{
                            name: 'help',
                            iconClass: 'umcSaveIconWhite',
                            label: _('Save'),
                            callback: lang.hitch(this, function() {
                                var vals = this._form.get("value");
                                this.saveSettings(vals);
                            }),
                        }],
                    });

                    this.addChild(this._mainPage);
                    this._mainPage.startup();

                    var widgets = [{
                        type: CheckBox,
                        name: 'enable',
                        label: _('Enable Kaspersky Mail Security'),
                        value: this._previousStatus,
                        onChange: lang.hitch(this, function() { this.updateChanged() }),
                    }, {
                        type: PasswordInputBox,
                        name: 'password',
                        description: _('Password for the webinterface'),
                        label: _('Password'),
                        onChange: lang.hitch(this, function() { this.updateChanged() }),
                    }, {
                        type: Text,
                        name: 'webinterface',
                        content: _('Open a new window with the <a href="/klms/" target="_blank">Kaspersky webinterface</a>. Use the username "Administrator" to login.'),
                    }];

                    var layout = [
                        { 
                            label: _("Global settings"),
                            layout: [ 'enable' ],
                        },
                        { 
                            label: _("Webinterface password"),
                            layout: [ 'password' ]
                        },
                        { 
                            label: _("Webinterface"),
                            layout: [ 'webinterface' ]
                        },

                    ];


                    // generate the search form
                    this._form = new Form({
                        // property that defines the widget's position in a dijit.layout.BorderContainer
                        region: 'top',
                        widgets: widgets,
                        layout: layout,
                    });

                    // turn off the standby animation as soon as all form values have been loaded
                    on.once(this._form, 'valuesInitialized', lang.hitch(this, function() {
                        this.standby(false);
                    }));

                    this._mainPage.addChild(this._form);
                }));

            },
            closeModule: function() {
                if (this._hasChanges) {
                    dialog.confirm( _( 'Should the UMC module be closed? All unsaved modification will be lost.' ), [ {
                        label: _( 'Close' ),
                        callback: lang.hitch( this, function() {
                            topic.publish('/umc/tabs/close', this );
                        } )
                    }, {
                        label: _( 'Cancel' ),
                        'default': true
                    } ] );
                }
                else {
                    topic.publish('/umc/tabs/close', this );
                }
            },

            saveSettings: function(data) {
                var message;

                if (data.enable != this._previousStatus) {
                    message = _('You are about to disable Kaspersky Mailsecurity. Do you want to continue?');

                    if (data.enable === true) {
                        message = _('You are about to enable Kaspersky Mailsecurity. This will also disable the current Amavis installation. Do you want to continue?');
                    }

                    this._previousStatus = data.enable;

                    dialog.confirm(message, [{
                        label: _('OK'),
                        callback: lang.hitch(this, function() {
                            this.standbyDuring( this.umcpCommand('klms/enable', data) );
                            this._hasChanges = false;
                        })
                    }, {
                        'default': true,
                        label: _('Cancel')
                    }]);
                }
                else {
                    this.standbyDuring( this.umcpCommand('klms/enable', data) );
                    this._hasChanges = false;
                }
            },
            updateChanged: function() {
                this._hasChanges = true;
            },
        });
    });
